@@ -13,15 +13,22 @@ use Devel::Peek;
 use Encode qw(encode_utf8 is_utf8);
 use Scalar::Util qw(reftype blessed refaddr);
 use Config;
+use Carp qw(confess);
 
 # Dynamically load constants from whatever is being tested
 our ($Class, $ConstClass);
 BEGIN {
-    if (-e "lib/Sereal/Encoder") {
+    if (-e "lib/Sereal/Encoder.pm") {
         $Class = 'Sereal::Encoder';
     }
-    elsif (-e "lib/Sereal/Decoder") {
+    elsif (-e "lib/Sereal/Decoder.pm") {
         $Class = 'Sereal::Decoder';
+    }
+    elsif (-e "lib/Sereal/Merger.pm") {
+        $Class = 'Sereal::Merger';
+    }
+    elsif (-e "lib/Sereal/Splitter.pm") {
+        $Class = 'Sereal::Splitter';
     } else {
         die "Could not find an applicable Sereal constants location";
     }
@@ -115,10 +122,21 @@ sub dump_bless {
 }
 
 sub short_string {
-    die if length($_[0]) > SRL_MASK_SHORT_BINARY_LEN;
-    my $tag = SRL_HDR_SHORT_BINARY_LOW + length($_[0]);
-    $tag |= SRL_HDR_TRACK_FLAG if $_[1];
-    return pack("c a*",$tag,$_[0]);
+    my ($str, $alias)= @_;
+    $alias ||= 0;
+    my $length= length($str);
+    if ($length > SRL_MASK_SHORT_BINARY_LEN) {
+        confess "String too long for short_string(), alias=$alias length=$length";
+    }
+    my $tag = SRL_HDR_SHORT_BINARY_LOW + length($str);
+    if ($tag > SRL_HDR_SHORT_BINARY_HIGH) {
+        confess "Tag value larger than SRL_HDR_SHORT_BINARY_HIGH, tag=$tag; alias=$alias; length=$length";
+    }
+    $tag |= SRL_HDR_TRACK_FLAG if $alias;
+    if ($tag > 255) {
+        confess "Tag value over 255 in short_string(), tag=$tag; alias=$alias; length=$length; SRL_HDR_TRACK_FLAG=", SRL_HDR_TRACK_FLAG;
+    }
+    return chr($tag) . $str;
 }
 
 sub integer {
@@ -234,14 +252,13 @@ sub setup_tests {
         [[], array(), "empty array ref"],
         [[1,2,3], array(chr(0b0000_0001), chr(0b0000_0010), chr(0b0000_0011)), "array ref"],
         [1000, chr(SRL_HDR_VARINT).varint(1000), "large int"],
-        [ [1..1000],
+        [ [ map { $_, undef } 1..1000 ],
             array(
-                (map chr, (1 .. SRL_POS_MAX_SIZE)),
-                (map chr(SRL_HDR_VARINT) . varint($_), ((SRL_POS_MAX_SIZE+1) .. 1000))
+                (map { chr($_) => chr(SRL_HDR_UNDEF) } (1 .. SRL_POS_MAX_SIZE)),
+                (map { chr(SRL_HDR_VARINT) . varint($_) => chr(SRL_HDR_UNDEF) } ((SRL_POS_MAX_SIZE+1) .. 1000))
             ),
-            "array ref with pos and varints"
+            "array ref with pos and varints and undef"
         ],
-
         [{}, hash(), "empty hash ref"],
         [{foo => "baaaaar"}, hash(short_string("foo"),short_string("baaaaar")), "simple hash ref"],
         [
@@ -572,7 +589,6 @@ sub have_encoder_and_decoder {
     # $Class is the already-loaded class, so the one we're testing
     my $need = $Class =~ /Encoder/ ? "Decoder" : "Encoder";
     my $need_class = "Sereal::$need";
-    my %compat_versions = map {$_ => 1} $Class->_test_compat();
 
     if (defined(my $top_dir = get_git_top_dir())) {
         my $blib_dir = File::Spec->catdir($top_dir, 'Perl', $need, "blib");
@@ -581,6 +597,11 @@ sub have_encoder_and_decoder {
             blib->import($blib_dir);
         }
     }
+    eval "use $Class; 1"
+    or do {
+        note("Could not locate $Class for testing" . ($@ ? " (Exception: $@)" : ""));
+        return();
+    };
 
     eval "use $need_class; 1"
     or do {
@@ -595,6 +616,7 @@ sub have_encoder_and_decoder {
     }
     $cmp_v =~ s/_//;
     $cmp_v = sprintf("%.2f", int($cmp_v*100)/100);
+    my %compat_versions = map {$_ => 1} $Class->_test_compat();
     if (not defined $cmp_v or not exists $compat_versions{$cmp_v}) {
         note("Could not load correct version of $need_class for testing "
              ."(got: $cmp_v, needed any of ".join(", ", keys %compat_versions).")");
